package kr.co.vividnext.sodalive.admin.calculate

import com.querydsl.core.annotations.QueryProjection
import java.math.BigDecimal
import java.math.RoundingMode

data class GetCalculateContentDonationQueryData @QueryProjection constructor(
    // 등록 크리에이터 닉네임
    val nickname: String,
    // 콘텐츠 제목
    val title: String,
    // 콘텐츠 가격
    val price: Int,
    // 콘텐츠 등록 날짜
    val registrationDate: String,
    // 후원 날짜
    val donationDate: String,
    // 인원
    val numberOfDonation: Long,
    // 합계
    val totalCan: Int
) {
    fun toGetCalculateContentDonationResponse(): GetCalculateContentDonationResponse {
        // 원화 = totalCoin * 100 ( 캔 1개 = 100원 )
        val totalKrw = BigDecimal(totalCan).multiply(BigDecimal(100))

        // 결제수수료 : 6.6%
        val paymentFee = totalKrw.multiply(BigDecimal(0.066))

        // 정산금액
        // 유료콘텐츠 (원화 - 결제수수료) 의 90%
        // 무료콘텐츠 (원화 - 결제수수료) 의 70%
        val settlementAmount = if (price > 0) {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(0.9))
        } else {
            totalKrw.subtract(paymentFee).multiply(BigDecimal(0.7))
        }

        // 원천세 = 정산금액의 3.3%
        val tax = settlementAmount.multiply(BigDecimal(0.033))

        // 입금액
        val depositAmount = settlementAmount.subtract(tax)

        val paidOrFree = if (price > 0) {
            "유료"
        } else {
            "무료"
        }

        return GetCalculateContentDonationResponse(
            nickname = nickname,
            title = title,
            paidOrFree = paidOrFree,
            registrationDate = registrationDate,
            donationDate = donationDate,
            numberOfDonation = numberOfDonation.toInt(),
            totalCan = totalCan,
            totalKrw = totalKrw.toInt(),
            paymentFee = paymentFee.setScale(0, RoundingMode.HALF_UP).toInt(),
            settlementAmount = settlementAmount.setScale(0, RoundingMode.HALF_UP).toInt(),
            tax = tax.setScale(0, RoundingMode.HALF_UP).toInt(),
            depositAmount = depositAmount.setScale(0, RoundingMode.HALF_UP).toInt()
        )
    }
}
