package kr.co.vividnext.sodalive.admin.member

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface AdminMemberRepository : JpaRepository<Member, Long>, AdminMemberQueryRepository

interface AdminMemberQueryRepository {
    fun getMemberTotalCount(role: MemberRole? = null): Int
    fun getMemberList(offset: Long, limit: Long, role: MemberRole? = null): List<Member>
    fun searchMember(searchWord: String, offset: Long, limit: Long, role: MemberRole? = null): List<Member>

    fun searchMemberTotalCount(searchWord: String, role: MemberRole? = null): Int
    fun getCreatorAllList(): List<GetAdminCreatorAllListResponse>
    fun findByIdAndActive(memberId: Long): Member?
}

class AdminMemberQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : AdminMemberQueryRepository {
    override fun getMemberList(offset: Long, limit: Long, role: MemberRole?): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(
                member.role.ne(MemberRole.ADMIN)
                    .and(
                        if (role != null) {
                            member.role.eq(role)
                        } else {
                            null
                        }
                    )
            )
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc())
            .fetch()
    }

    override fun getMemberTotalCount(role: MemberRole?): Int {
        return queryFactory
            .select(member.id)
            .from(member)
            .where(
                member.role.ne(MemberRole.ADMIN)
                    .and(
                        if (role != null) {
                            member.role.eq(role)
                        } else {
                            null
                        }
                    )
            )
            .fetch()
            .size
    }

    override fun searchMember(searchWord: String, offset: Long, limit: Long, role: MemberRole?): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(
                member.nickname.contains(searchWord)
                    .or(member.email.contains(searchWord))
                    .and(
                        if (role != null) {
                            member.role.eq(role)
                        } else {
                            null
                        }
                    )
            )
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc())
            .fetch()
    }

    override fun searchMemberTotalCount(searchWord: String, role: MemberRole?): Int {
        return queryFactory
            .select(member.id)
            .from(member)
            .where(
                member.nickname.contains(searchWord)
                    .or(member.email.contains(searchWord))
                    .and(
                        if (role != null) {
                            member.role.eq(role)
                        } else {
                            null
                        }
                    )
            )
            .fetch()
            .size
    }

    override fun getCreatorAllList(): List<GetAdminCreatorAllListResponse> {
        return queryFactory
            .select(
                QGetAdminCreatorAllListResponse(
                    member.id,
                    member.nickname
                )
            )
            .from(member)
            .where(
                member.role.eq(MemberRole.CREATOR)
                    .and(member.isActive.isTrue)
            )
            .fetch()
    }

    override fun findByIdAndActive(memberId: Long): Member? {
        return queryFactory
            .selectFrom(member)
            .where(
                member.id.eq(memberId)
                    .and(member.isActive.isTrue)
            )
            .orderBy(member.id.desc())
            .fetchFirst()
    }
}
