package kr.co.vividnext.sodalive.audition.applicant

import com.amazonaws.services.s3.model.ObjectMetadata
import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.audition.AuditionApplicant
import kr.co.vividnext.sodalive.audition.role.AuditionRoleRepository
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AuditionApplicantService(
    private val repository: AuditionApplicantRepository,
    private val roleRepository: AuditionRoleRepository,

    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    fun getAuditionApplicantList(
        auditionRoleId: Long,
        sortType: AuditionApplicantSortType,
        offset: Long,
        limit: Long
    ): GetAuditionApplicantListResponse {
        val totalCount = repository.getAuditionApplicantTotalCount(auditionRoleId = auditionRoleId)
        val items = repository.getAuditionApplicantList(
            auditionRoleId = auditionRoleId,
            sortType = sortType,
            offset = offset,
            limit = limit
        )

        return GetAuditionApplicantListResponse(
            totalCount = totalCount,
            items = items
        )
    }

    @Transactional
    fun applyAuditionRole(contentFile: MultipartFile?, requestString: String, member: Member) {
        if (contentFile == null) throw SodaException("녹음 파일을 확인해 주세요.")
        val request = objectMapper.readValue(requestString, ApplyAuditionRoleRequest::class.java)

        val auditionRole = roleRepository.findByIdOrNull(id = request.roleId)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

        val existingApplicant = repository.findActiveApplicantByMemberIdAndRoleId(
            memberId = member.id!!,
            roleId = auditionRole.id!!
        )

        if (existingApplicant != null) {
            existingApplicant.isActive = false
            repository.save(existingApplicant)
        }

        val applicant = AuditionApplicant(phoneNumber = request.phoneNumber)
        applicant.role = auditionRole
        applicant.member = member
        repository.save(applicant)

        val contentFileName = generateFileName(prefix = "${applicant.id}-applicant")

        val metadata = ObjectMetadata()
        metadata.contentLength = contentFile.size

        val contentPath = s3Uploader.upload(
            inputStream = contentFile.inputStream,
            bucket = bucket,
            filePath = "audition/applicant/${applicant.id}/$contentFileName",
            metadata = metadata
        )
        applicant.voicePath = contentPath
    }
}
