package kr.co.vividnext.sodalive.audition.vote

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.audition.AuditionVote
import kr.co.vividnext.sodalive.audition.QAuditionVote.auditionVote
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface AuditionVoteRepository : JpaRepository<AuditionVote, Long>, AuditionVoteQueryRepository

interface AuditionVoteQueryRepository {
    fun countByMemberIdAndVoteDateRange(
        memberId: Long,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int
}

class AuditionVoteQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AuditionVoteQueryRepository {
    override fun countByMemberIdAndVoteDateRange(
        memberId: Long,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int {
        return queryFactory
            .select(auditionVote.id)
            .from(auditionVote)
            .where(
                auditionVote.member.id.eq(memberId)
                    .and(auditionVote.createdAt.between(startDate, endDate))
            )
            .fetch()
            .size
    }
}
