package kr.co.vividnext.sodalive.can.coupon

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.coupon.QCanCoupon.canCoupon
import kr.co.vividnext.sodalive.can.coupon.QCanCouponNumber.canCouponNumber
import org.springframework.data.jpa.repository.JpaRepository

interface CanCouponNumberRepository : JpaRepository<CanCouponNumber, Long>, CanCouponNumberQueryRepository

interface CanCouponNumberQueryRepository {
    fun getUseCouponCount(id: Long): Int

    fun getCouponNumberTotalCount(couponId: Long): Int

    fun getCouponNumberList(couponId: Long, offset: Long, limit: Long): List<GetCouponNumberListItemResponse>

    fun getAllCouponNumberList(couponId: Long): List<GetCouponNumberListItemResponse>

    fun findByCouponNumber(couponNumber: String): CanCouponNumber?

    fun findByMemberId(memberId: Long): List<Long>
}

class CanCouponNumberQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CanCouponNumberQueryRepository {
    override fun getUseCouponCount(id: Long): Int {
        return queryFactory
            .select(canCouponNumber.id)
            .from(canCouponNumber)
            .innerJoin(canCouponNumber.canCoupon, canCoupon)
            .where(
                canCouponNumber.member.isNotNull
                    .and(canCouponNumber.canCoupon.id.eq(id))
            )
            .fetch()
            .size
    }

    override fun getCouponNumberTotalCount(couponId: Long): Int {
        return queryFactory
            .select(canCouponNumber.id)
            .from(canCouponNumber)
            .where(canCouponNumber.canCoupon.id.eq(couponId))
            .fetch()
            .size
    }

    override fun getCouponNumberList(couponId: Long, offset: Long, limit: Long): List<GetCouponNumberListItemResponse> {
        return queryFactory
            .select(
                QGetCouponNumberListItemResponse(
                    canCouponNumber.id,
                    canCouponNumber.couponNumber,
                    canCouponNumber.member.isNotNull
                )
            )
            .from(canCouponNumber)
            .where(canCouponNumber.canCoupon.id.eq(couponId))
            .orderBy(canCouponNumber.id.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getAllCouponNumberList(couponId: Long): List<GetCouponNumberListItemResponse> {
        return queryFactory
            .select(
                QGetCouponNumberListItemResponse(
                    canCouponNumber.id,
                    canCouponNumber.couponNumber,
                    canCouponNumber.member.isNotNull
                )
            )
            .from(canCouponNumber)
            .where(canCouponNumber.canCoupon.id.eq(couponId))
            .orderBy(canCouponNumber.id.asc())
            .fetch()
    }

    override fun findByCouponNumber(couponNumber: String): CanCouponNumber? {
        return queryFactory
            .selectFrom(canCouponNumber)
            .where(canCouponNumber.couponNumber.eq(couponNumber))
            .fetchFirst()
    }

    override fun findByMemberId(memberId: Long): List<Long> {
        return queryFactory
            .select(canCouponNumber.id)
            .from(canCouponNumber)
            .where(canCouponNumber.member.id.eq(memberId))
            .fetch()
    }
}
