package kr.co.vividnext.sodalive.content.order

import kr.co.vividnext.sodalive.common.BaseEntity
import kr.co.vividnext.sodalive.content.AudioContent
import kr.co.vividnext.sodalive.member.Member
import java.time.LocalDateTime
import javax.persistence.Entity
import javax.persistence.EnumType
import javax.persistence.Enumerated
import javax.persistence.FetchType
import javax.persistence.JoinColumn
import javax.persistence.ManyToOne
import javax.persistence.Table
import kotlin.math.ceil

enum class OrderType {
    RENTAL, KEEP
}

@Entity
@Table(name = "orders")
data class Order(
    @Enumerated(value = EnumType.STRING)
    val type: OrderType,
    var isActive: Boolean = true
) : BaseEntity() {
    var can: Int = 0

    val startDate: LocalDateTime = LocalDateTime.now()
    var endDate: LocalDateTime? = null

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "creator_id", nullable = false)
    var creator: Member? = null

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "member_id", nullable = false)
    var member: Member? = null

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "content_id", nullable = false)
    var audioContent: AudioContent? = null
        set(value) {
            can = if (type == OrderType.RENTAL && !value?.isOnlyRental!!) {
                ceil(value.price * 0.7).toInt()
            } else {
                value?.price!!
            }
            field = value
        }

    override fun prePersist() {
        super.prePersist()
        if (type == OrderType.RENTAL) {
            endDate = startDate.plusDays(5)
        }
    }
}
