package kr.co.vividnext.sodalive.live.room

import kr.co.vividnext.sodalive.can.use.UseCan
import kr.co.vividnext.sodalive.common.BaseEntity
import kr.co.vividnext.sodalive.live.reservation.LiveReservation
import kr.co.vividnext.sodalive.live.room.cancel.LiveRoomCancel
import kr.co.vividnext.sodalive.live.room.visit.LiveRoomVisit
import kr.co.vividnext.sodalive.member.Member
import java.time.LocalDateTime
import javax.persistence.CascadeType
import javax.persistence.Column
import javax.persistence.Entity
import javax.persistence.EnumType
import javax.persistence.Enumerated
import javax.persistence.FetchType
import javax.persistence.JoinColumn
import javax.persistence.OneToMany
import javax.persistence.OneToOne

@Entity
data class LiveRoom(
    var title: String,
    @Column(columnDefinition = "TEXT", nullable = false)
    var notice: String,
    var beginDateTime: LocalDateTime,
    var numberOfPeople: Int,
    var coverImage: String? = null,
    var bgImage: String? = null,
    var isAdult: Boolean,
    val price: Int = 0,
    val isAvailableJoinCreator: Boolean = true,
    @Enumerated(value = EnumType.STRING)
    val type: LiveRoomType = LiveRoomType.OPEN,
    @Column(nullable = true)
    var password: String? = null
) : BaseEntity() {
    @OneToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "member_id", nullable = false)
    var member: Member? = null

    @OneToMany(mappedBy = "room", cascade = [CascadeType.ALL])
    var tags: MutableList<LiveRoomTag> = mutableListOf()

    @OneToMany(mappedBy = "room", fetch = FetchType.LAZY)
    var useCan: MutableList<UseCan> = mutableListOf()

    @OneToMany(mappedBy = "room", fetch = FetchType.LAZY)
    var reservations: MutableList<LiveReservation> = mutableListOf()

    @OneToMany(mappedBy = "room", fetch = FetchType.LAZY)
    var visits: MutableList<LiveRoomVisit> = mutableListOf()

    @OneToOne(mappedBy = "room")
    var cancel: LiveRoomCancel? = null
    var channelName: String? = null
    var isActive: Boolean = true
}

enum class LiveRoomType {
    // 공개
    OPEN,

    // 비공개
    PRIVATE
}

enum class LiveRoomStatus {
    NOW, RESERVATION
}
