package kr.co.vividnext.sodalive.marketing

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.springframework.stereotype.Service

@Service
class AdTrackingService(
    private val repository: AdTrackingRepository,
    private val mediaPartnerRepository: AdMediaPartnerRepository
) {

    private val coroutineScope = CoroutineScope(Dispatchers.IO)

    fun saveTrackingHistory(
        pid: String,
        type: AdTrackingHistoryType,
        memberId: Long,
        price: Double? = null,
        locale: String? = null
    ) {
        coroutineScope.launch {
            try {
                val mediaPartner = mediaPartnerRepository.findByPid(pid)

                if (mediaPartner != null) {
                    val id = AdTrackingHistoryId(pid = pid, memberId = memberId, type = type)
                    val trackingHistory = AdTrackingHistory(
                        id = id,
                        mediaGroup = mediaPartner.mediaGroup,
                        pidName = mediaPartner.pidName,
                        price = price ?: 0.toDouble(),
                        locale = locale
                    )
                    repository.save(trackingHistory)
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }
}
